﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Modbus.Device;

namespace ehmedemo
{
    class Comm
    {
        private static SerialPort serialPort;
        private static ModbusSerialMaster master;
        
        public static SerialPort GetSerialPort()
        {
            if (serialPort == null)
            {
                serialPort = new SerialPort();
            }
            return serialPort;
        }

        public static string[] GetPorts()
        {
            string[] ports = SerialPort.GetPortNames();
            Array.Sort(ports);
            return ports;
        }

        public static ModbusSerialMaster GetModbus()
        {
            if (master == null)
            {
                master = ModbusSerialMaster.CreateRtu(GetSerialPort());
                master.Transport.ReadTimeout = 100;
                master.Transport.WriteTimeout = 100;
                master.Transport.Retries = 0;
                master.Transport.WaitToRetryMilliseconds = 100;
            }
            return master;
        }

        public static bool GetStatus()
        {
            return GetSerialPort().IsOpen;
        }

        public static bool OpenPort()
        {
            try
            {
                GetSerialPort().PortName = Param.Port;
                GetSerialPort().BaudRate = Param.Baud;
                GetSerialPort().Parity = GetParity(Param.Parity);
                GetSerialPort().DataBits = 8;
                GetSerialPort().StopBits = StopBits.One;
                GetSerialPort().Open();
                return true;
            }
            catch
            {
                return false;
            }
        }

        private static Parity GetParity(int parity)
        {
            Parity p;
            switch (parity)
            {
                case 0:
                default:
                    p = Parity.None;
                    break;
                case 1:
                    p = Parity.Odd;
                    break;
                case 2:
                    p = Parity.Even;
                    break;
                case 3:
                    p = Parity.Space;
                    break;
            }
            return p;
        }

        public static void ClosePort()
        {
            GetSerialPort().Close();
        }
        
        public static ushort[] ReadInputRegisters(ushort address, ushort length)
        {
            try
            {
                return GetModbus().ReadInputRegisters(Param.Addr, address, length);
            }
            catch(Exception e)
            {
                Console.WriteLine(e.Message);
            }
            return null;
        }

        public static ushort[] ReadHoldingRegisters(ushort address, ushort length)
        {
            try
            {
                return GetModbus().ReadHoldingRegisters(Param.Addr, address, length);
            }
            catch(Exception e)
            {
                Console.WriteLine(e.Message);
            }
            return null;
        }

        public static bool WriteHoldingRegisters(ushort address, ushort[] data)
        {
            try
            {
                GetModbus().WriteMultipleRegisters(Param.Addr, address, data);
                return true;
            }
            catch(Exception e)
            {
                Console.WriteLine(e.Message);
            }
            return false;
        }
    }
}
