﻿
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net.NetworkInformation;
using System.Text;
using System.Threading;
using System.Timers;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using Color = System.Drawing.Color;

namespace ehmedemo
{
    public partial class FormMain : Form
    {
       
        private bool isClosing = false;
        private bool isConnected = false;

        private System.Timers.Timer statusTimer = null;
        private System.Timers.Timer valueTimer = null;

        private Info info;
        private Item item;

        private string defUnit;

        public FormMain()
        {
            InitializeComponent();

            InitParam();
            InitForm();
            InitPort();
            InitTimer();
        }

        private void InitParam()
        {
            isConnected = false;
            defUnit = "";
            info = null;
            item = null;
            Common.SetText(tb_current, "------");
        }

        private void InitTimer()
        {
            statusTimer = new System.Timers.Timer();
            statusTimer.AutoReset = true;
            statusTimer.Interval = 50;
            statusTimer.Elapsed += StatusTimer_Elapsed;
            statusTimer.Start();

            valueTimer = new System.Timers.Timer();
            valueTimer.AutoReset = false;
            valueTimer.Elapsed += ValueTimer_Elapsed;
            valueTimer.Start();
        }

        private void StatusTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (isClosing)
                return;

            Common.SetForeColor(lbl_portStatus, Comm.GetStatus() ? (isConnected ? Color.Green : Color.Red) : Color.Gray);
        }

        private void readInfo()
        {
            ushort[] value = Comm.ReadInputRegisters(Info.START_ADDR, (ushort)Info.READ_LENGTH);
            if (value != null && value.Length == Info.READ_LENGTH)
            {
                info = new Info();
                info.SetValue(value);
            }
            else
            {
                InitParam();
            }
        }

        private void readItem()
        {
            ushort[] value = Comm.ReadInputRegisters(Item.START_ADDR, info.IsNewValueType() ? (ushort)Item.READ_LENGTH_NEW : (ushort)Item.READ_LENGTH);
            if (value != null && info.IsNewValueType() ? value.Length == Item.READ_LENGTH_NEW : value.Length == Item.READ_LENGTH)
            {
                item = new Item();
                item.SetValue(value);
                isConnected = true;

                readDefUnit();
            }
            else
            {
                InitParam();
            }
        }

        private void readDefUnit()
        {
            if (info.IsNewValueType())
            {
                ushort[] value = Comm.ReadHoldingRegisters(17, 2);
                if (value != null)
                {
                    byte[] defBytes = new byte[6];
                    defBytes[0] = (byte)((value[0] >> 8) & 0xFF);
                    defBytes[1] = (byte)((value[0] >> 0) & 0xFF);
                    defBytes[2] = (byte)((value[1] >> 8) & 0xFF);
                    defBytes[3] = (byte)((value[1] >> 0) & 0xFF);
                    defUnit = Encoding.GetEncoding("GB18030").GetString(defBytes).Trim(); ;
                }
            }
            else
            {
                defUnit = "";
            }
        }

        private void ValueTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (isClosing)
                return;

            if (Comm.GetStatus())
            {
                try
                {
                    //读取设备型号
                    readInfo();

                    Common.Sleep(5);
                    if(info != null)
                    {
                        //读取设备数据
                        readItem();
                        Common.Sleep(5);

                        if(item != null)
                        {
                            //读取ME1XX测量单位
                            readDefUnit();
                        }
                    }

                    if(item != null)
                    {
                        //当前数据
                        long current = item.Value;
                        //当前单位
                        int unit = info.IsNewValueType() ? item.NewUnit : item.Unit;
                        //当前小数点位数
                        int point = info.IsNewValueType() ? item.Point : item.Unit;
                        //当前标志位
                        Flag flag = item.getFlag();

                        if(flag.Overload)
                        {
                            Common.SetText(tb_current, "超量程");
                        }
                        else
                        {
                            Common.SetText(tb_current, GetRealValueAndUnit(current, unit, point, defUnit));
                        }                        
                    }                    

                }
                catch
                {
                    InitParam();
                }
            } 
            else
            {
                InitParam();
            }

            Common.Sleep(5);
            valueTimer.Start();
        } 

        private string GetRealValueAndUnit(long value, int unit, int point, string defUnit)
        {
            return Common.AddPoint(value, Item.UNIT_ITEM_POINT[Item.GetUnitIndex(info)][point]) + Item.GetUnitString(info, unit, defUnit);
        }

        private void InitForm()
        {
            this.Text = "ME系列DEMO" + "  ——  " + " Ver " + "1.00";

            cb_baud.Items.AddRange(Param.PORT_BAUD_ITEM);
            cb_parity.Items.AddRange(Param.COMU_PARITY_ITEM);
            cb_baud.SelectedItem = Param.Baud.ToString();
            cb_parity.SelectedIndex = Param.Parity;
            tb_addr.Text = Param.Addr.ToString();
        }

        private void InitPort()
        {
            cb_port.Items.Clear();
            string[] ports = Comm.GetPorts();
            if (ports == null || ports.Length == 0)
            {
                cb_port.Items.Add("COM1");
            }
            else
            {
                cb_port.Items.AddRange(ports);
            }
            cb_port.SelectedItem = Param.Port;
            if (cb_port.SelectedIndex == -1)
            {
                cb_port.SelectedIndex = 0;
            }
        }

        private void btn_openPort_Click(object sender, EventArgs e)
        {
            if (Comm.GetStatus())
            {
                Comm.ClosePort();
                btn_openPort.Text = "打开";
            }
            else
            {
                SetPort();
                if (Comm.OpenPort())
                {
                    btn_openPort.Text = "关闭";
                }
                else
                {
                    btn_openPort.Text = "打开";
                    MessageBox.Show("串口打开失败");
                }
            }
            InitParam();
        }

        private void SetPort()
        {
            Param.Port = cb_port.SelectedItem.ToString();
            Param.Baud = Convert.ToInt32(cb_baud.SelectedItem);
            Param.Parity = cb_parity.SelectedIndex;
            try
            {
                Param.Addr = Convert.ToByte(tb_addr.Text);
            }
            catch
            {
                Param.Addr = 1;
                tb_addr.Text = "1";
            }
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            isClosing = true;
            Thread.Sleep(20);
        }

    }
}
