﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ehmedemo
{
    class Item : ICloneable
    {
        public static readonly ushort START_ADDR = 0;
        public static readonly int READ_LENGTH = 13;
        public static readonly int READ_LENGTH_NEW = 13;

        public static readonly int UNIT_INDEX_01X = 0;
        public static readonly int UNIT_INDEX_02X = 1;
        public static readonly int UNIT_INDEX_1XX = 2;

        public static readonly int CHAN_30V_010 = 0;
        public static readonly int CHAN_4V_010 = 1;
        public static readonly int CHAN_300MA_010 = 2;
        public static readonly int CHAN_30MA_010 = 3;

        public static readonly string[] UNIT_ITEM_010 = {"0.00001V", "0.000001V", "0.0001mA", "0.00001mA"};
        public static readonly int[] UNIT_ITEM_POINT_010 = {5, 6, 4, 5};
        public static readonly string[] UNIT_ITEM_UNIT_010 = {"V", "V", "mA", "mA"};

        public static readonly string[] UNIT_ITEM_020 = {"0.001mΩ", "0.00001Ω", "0.0001Ω", "0.001Ω", "0.00001kΩ", "0.0001kΩ", "0.01℃"};
        public static readonly int[] UNIT_ITEM_POINT_020 = {3, 5, 4, 3, 5, 4, 2};
        public static readonly string[] UNIT_ITEM_UNIT_020 = {"mΩ", "Ω", "Ω", "Ω", "kΩ", "kΩ", "℃"};

        public static readonly string[] UNIT_ITEM_1XX = { "" };
        public static readonly int[] UNIT_ITEM_POINT_1XX = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
        public static readonly string[] UNIT_ITEM_UNIT_1XX = { 
            " ",     //0  空
            "DEF",   //1  自定义
            "ug",    //2 
            "mg",    //3
            "g",     //4
            "kg",    //5
            "t",     //6
            "lb",    //7
            "uΩ",   //8 
            "mΩ",   //9
            "Ω",    //10
            "kΩ",   //11 
            "MΩ",   //12 
            "℃",    //13
            "℉",    //14
            "uV",    //15 
            "mV",    //16
            "V",     //17
            "kV",    //18
            "uA",    //19
            "mA",    //20
            "A",     //21
            "kA",    //22
            "um",    //23
            "mm",    //24
            "cm",    //25
            "dm",    //26
            "m",     //27
            "km",    //28
            "mL",    //29
            "L",     //30
            "s"      //31 
        };

        public static readonly string[][] UNIT_ITEM = {UNIT_ITEM_010, UNIT_ITEM_020, UNIT_ITEM_1XX };
        public static readonly int[][] UNIT_ITEM_POINT = {UNIT_ITEM_POINT_010, UNIT_ITEM_POINT_020, UNIT_ITEM_POINT_1XX };
        public static readonly string[][] UNIT_ITEM_UNIT = {UNIT_ITEM_UNIT_010, UNIT_ITEM_UNIT_020, UNIT_ITEM_UNIT_1XX };

        public static readonly int VALUE_OVERLOAD = 0x7FFFFFFF;
            
        public long Value { get; set; }
        public int Unit { get; set; }
        public int Sps { get; set; }
        public int Depth { get; set; }
        public int Ad { get; set; }
        public short Temp { get; set; }
        public int Softver { get; set; }

        public int Point { get; set; }
        public int NewUnit { get; set; }
        public int flag { get; set; }


        public Item()
        {
            Value = 0;
            Unit = 0;
            Sps = 0;
            Depth = 0;
            Point = 0;
            NewUnit = 0;
            flag = 0;
        }

        public void SetValue(ushort[] value)
        {
            Value = (value[0] << 16) | value[1];
            Unit = value[2];
            Sps = value[3];
            Depth = value[4];
            Ad = (value[5] << 16) | value[6];
            Temp = (short)value[7];
            Softver = value[8];

            Point = value[10];
            NewUnit = value[11];
            flag = value[12];
        }

        public Flag getFlag()
        {
            return new Flag(flag);
        }

        public string GetTemp()
        {
            return string.Format("{0:#.0}", Math.Round(Temp /2d, 1, MidpointRounding.AwayFromZero));
        }

        public string GetSoftver()
        {
            return Common.AddPoint(Softver & 0x7FFF, 2);
        }

        public bool IsDecrypted()
        {
            return (Softver & 0x8000) == 0;
        }

        public static int GetUnitIndex(Info info)
        {
            if (info != null)
            {
                if (info.Category == Info.CATEGORY_ME && (info.Model == Info.MODEL_ME_010 || info.Model == Info.MODEL_ME_011))
                {
                    return UNIT_INDEX_01X;
                }
                if (info.Category == Info.CATEGORY_ME && info.Model == Info.MODEL_ME_020)
                {
                    return UNIT_INDEX_02X;
                }
                if (info.Category == Info.CATEGORY_ME && info.Model == Info.MODEL_ME_1XX)
                {
                    return UNIT_INDEX_1XX;
                }
            }
            return 0;
        }

        public static string GetUnitPointString(Info info, int lastUnit, int lastPoint, string lastDefUnit)
        {
            if (info != null)
            {
                if (info.Category == Info.CATEGORY_ME && (info.Model == Info.MODEL_ME_010 || info.Model == Info.MODEL_ME_011))
                {
                    return UNIT_ITEM[GetUnitIndex(info)][lastUnit];
                }
                if (info.Category == Info.CATEGORY_ME && info.Model == Info.MODEL_ME_020)
                {
                    return UNIT_ITEM[GetUnitIndex(info)][lastUnit];
                }
                if (info.Category == Info.CATEGORY_ME && info.Model == Info.MODEL_ME_1XX)
                {
                    return Common.AddPoint(1, lastPoint) + GetUnitString(info, lastUnit, lastDefUnit);
                }
            }
            return "";
        }

        public static string GetUnitString(Info info, int lastUnit, string lastDefUnit)
        {
            if (info != null)
            {
                if (info.Category == Info.CATEGORY_ME && (info.Model == Info.MODEL_ME_010 || info.Model == Info.MODEL_ME_011))
                {
                    return UNIT_ITEM_UNIT[GetUnitIndex(info)][lastUnit];
                }
                if (info.Category == Info.CATEGORY_ME && info.Model == Info.MODEL_ME_020)
                {
                    return UNIT_ITEM_UNIT[GetUnitIndex(info)][lastUnit];
                }
                if (info.Category == Info.CATEGORY_ME && info.Model == Info.MODEL_ME_1XX)
                {
                    if(lastUnit == 1)
                    {
                        return lastDefUnit;
                    }
                    else
                    {
                        return UNIT_ITEM_UNIT[GetUnitIndex(info)][lastUnit];
                    }
                }
            }
            return "";
        }

        public object Clone()
        {
            return MemberwiseClone();
        }
    }
}
